/*!
 * ImmobilienScout24 Core CSS Framework, v1.2.0
 * http://www.immobilienscout24.de/
 */
var IS24 = IS24 || {};
IS24.core = IS24.core || {};

/* --- Helper Functions for Core Framework Scripts --- */

IS24.core.helpers = {
	documentReady: function(callback) {
		if (document.addEventListener) {
			document.addEventListener('DOMContentLoaded', callback);
		} else if (document.attachEvent) {
			document.attachEvent('onreadystatechange', function() {
				if (document.readyState === 'complete') {
					callback();
				}
			});
		}
	},
	
	addEventListener: function (targetElement, eventType, handler, useCapture) {
		if (targetElement.addEventListener) {
			targetElement.addEventListener(eventType, handler, useCapture);
			return true;
		} else if (targetElement.attachEvent) {
			return targetElement.attachEvent("on" + eventType, handler);
		}
	},
	
	getPageScrollTop: function () {
		var scrollTop;
		if ("scrollTop" in document.documentElement) {
			// get sum of document and body offset so it doesn't matter which of the two the browser scrolls - either one should be zero
			scrollTop = document.documentElement.scrollTop + document.body.scrollTop;
		} else if ("pageYOffset" in document.body) {
			scrollTop = document.body.pageYOffset;
		}
		return scrollTop;
	},
	
	getCoordinates: function (element) {
		var x = 0,
			y = 0;
		while(element) {
			x += element.offsetLeft;
			y += element.offsetTop;
			element = element.offsetParent;
		}
		return { left: x, top: y };
	},
	
	hasClass: function (element, wantedClass) {
		var classString,
			classMatch;
		if (!element || !wantedClass) { return; }
		classString = (" " + element.className + " ").replace(/\s+/g, " "); // makes sure all class names are padded by single white spaces
		classMatch = new RegExp("\\s" + wantedClass + "\\s", "g");
		return classString.match(classMatch);
	},
	
	addClass: function (element, addedClass) {
		var elementHasClass = this.hasClass(element, addedClass);
		if (!elementHasClass) {
			element.className += " " + addedClass;
		}
	},
	
	removeClass: function (element, removedClass) {
		var elementHasClass = this.hasClass(element, removedClass);
		if (elementHasClass) {
			element.className = (" " + element.className + " ").replace(" " + removedClass + " ", " ").replace(/^\s+|\s+$/g, ''); // remove class, trim
		}
	},
	
	toggleClass: function (element, toggledClass) {
		var elementHasClass = this.hasClass(element, toggledClass);
		if (elementHasClass) {
			this.removeClass(element, toggledClass);
		} else {
			this.addClass(element, toggledClass);
		}
	},
	
	nearestAncestor: function (element, ancestorSelector) {
		var currentNode = element,
			matchedAncestor,
			isClassSelector = (ancestorSelector.indexOf(".") === 0);
		
		ancestorSelector = (isClassSelector) ? ancestorSelector.substring(1) : ancestorSelector;
		
		if (isClassSelector) {
			while (currentNode.parentNode) {
				currentNode = currentNode.parentNode;
				if (this.hasClass(currentNode, ancestorSelector)) {
					matchedAncestor = currentNode;
					break;
				}
			}
		} else {
			while (currentNode.parentNode) {
				currentNode = currentNode.parentNode;
				if (currentNode.nodeName.toLowerCase() === ancestorSelector.toLowerCase()) {
					matchedAncestor = currentNode;
					break;
				}
			}
		}
		
		return matchedAncestor;
	}
};
/* --- Fixed Elements --- */
/*
 *  Will make an element stick to the top of the window when the browser scrolls past it.
 *  Initiated through "IS24.core.fixElementToViewportTop.init()".
 *  Required Markup:
 *  <div class="fix-to-top-wrapper">
 *      <div class="fix-to-top"></div>
 *  </div>
 */

IS24.core.fixElementToViewportTop = (function () {
	var fixedElementWrapper,
		fixedElement,
		isFixedClass,
		fixedPoint,
		currentOffset,
		isFixed = false,
		throttle,
		throttleTimeout = 1000;
	
	function updateCurrentOffset() {
		currentOffset = IS24.core.helpers.getPageScrollTop();
	}
	function updateFixedPoint() {
		fixedPoint = IS24.core.helpers.getCoordinates(fixedElementWrapper).top;
	}
	
	function toggleFixed() {
		if (currentOffset >= fixedPoint && !isFixed) {
			IS24.core.helpers.addClass(fixedElement, isFixedClass);
			isFixed = true;
		} else if (currentOffset < fixedPoint && isFixed) {
			IS24.core.helpers.removeClass(fixedElement, isFixedClass);
			isFixed = false;
		}
	}
	
	// selective scroll handler to reduce computing time if scroll events are fired in rapid succession
	function throttledHandler() {
		if (!throttle) {
			throttle = window.setTimeout(function () {
				throttle = null;
			}, throttleTimeout);
			
			// put stuff here that should only run every so often
			updateFixedPoint();
		}
		// put stuff here that needs to run every time for responsiveness
		updateCurrentOffset();
		toggleFixed();
	}
	
	function init() {
		fixedElementWrapper = document.querySelector(".fix-to-top-wrapper");
		fixedElement = document.querySelector(".fix-to-top");
		isFixedClass = "fix-to-top--fixed";
		
		if (!fixedElementWrapper || !fixedElement) { return; }
		
		IS24.core.helpers.addEventListener(window, "resize", throttledHandler);
		IS24.core.helpers.addEventListener(window, "scroll", throttledHandler);
		
		// initial execution
		updateFixedPoint();
		updateCurrentOffset();
		toggleFixed();
	}
	
	return { init: init };
}());

/* --- Top Navigation --- */

IS24.core.topNavigation = {
	slideIn: (function () {
		var leftTrigger,
			rightTrigger,
			slideInContainer,
			slideInContainerIsOpenedClassFragment;
		
		function clickHandler(event, direction) {
			event.preventDefault();
			IS24.core.helpers.toggleClass(slideInContainer, slideInContainerIsOpenedClassFragment + direction);
		}
        
        function documentHandler(event) {
            var target = event.target || event.srcElement;
            if (IS24.core.helpers.nearestAncestor(target, ".top-navigation__slide-in--left") ||
                IS24.core.helpers.nearestAncestor(target, ".top-navigation__slide-in--right")) {
                event.stopPropagation();
            }
            else if (IS24.core.helpers.hasClass(slideInContainer, slideInContainerIsOpenedClassFragment + "left")) {
                event.preventDefault();
                IS24.core.helpers.removeClass(slideInContainer, slideInContainerIsOpenedClassFragment + "left");
            }
            else if (IS24.core.helpers.hasClass(slideInContainer, slideInContainerIsOpenedClassFragment + "right")) {
                event.preventDefault();
                IS24.core.helpers.removeClass(slideInContainer, slideInContainerIsOpenedClassFragment + "right");
            }
        }
		
		function init() {
			leftTrigger = document.querySelector(".top-navigation__slide-in-trigger--left");
			rightTrigger = document.querySelector(".top-navigation__slide-in-trigger--right");
			slideInContainer = document.querySelector(".top-navigation-slide-in-container");
			slideInContainerIsOpenedClassFragment = "top-navigation-slide-in-container--open-";
			
			if (!slideInContainer) { return; }
			
			if (leftTrigger) {
				IS24.core.helpers.addEventListener(leftTrigger, "click", function (event) {
					clickHandler(event, "left");
				});
			}
			if (rightTrigger) {
				IS24.core.helpers.addEventListener(rightTrigger, "click", function (event) {
					clickHandler(event, "right");
				});
			}
            if (leftTrigger || rightTrigger) {
                IS24.core.helpers.addEventListener(document, "touchend", function (event) {
                    documentHandler(event);
                });
            }
		}
		
		return { init: init };
	}()),
	
	touchHover: (function () {
		var hoverTabs,
			hoverTabLabels,
			pseudoHoverClass,
			lastTouchedItem,
			deskBreakPoint = 1014;
		
		// prevent first click and simulate hover instead
		function tabHandler(event) {
			var ancestorLIElement,
				i;
			
			if (!window.matchMedia("(min-width: " + deskBreakPoint + "px)").matches) { return; } // skip touch handling when only mobile-nav is displayed
			
			if (lastTouchedItem !== event.target) {
				lastTouchedItem = event.target;
				event.preventDefault();
				event.stopPropagation();
				for (i=0; i<hoverTabs.length; i++) {
					IS24.core.helpers.removeClass(hoverTabs[i], pseudoHoverClass);
				}
				ancestorLIElement = IS24.core.helpers.nearestAncestor(event.target, "li");
				IS24.core.helpers.addClass(ancestorLIElement, pseudoHoverClass);
			}
		}
		
		// remove hover effect on touch events outside of layer
		function documentHandler() {
			var i;
			
			if (!lastTouchedItem) { return; }
			
			lastTouchedItem = undefined;
			for (i=0; i<hoverTabs.length; i++) {
				IS24.core.helpers.removeClass(hoverTabs[i], pseudoHoverClass);
			}
		}
		
		function attachEventListeners() {
			var i;
			for (i=0; i<hoverTabLabels.length; i++) {
				IS24.core.helpers.addEventListener(hoverTabLabels[i], "touchstart", tabHandler);
			}
			IS24.core.helpers.addEventListener(document, "touchstart", documentHandler);
		}
		
		function init() {
			hoverTabs = document.querySelectorAll(".top-navigation__hover-tabs > li");
			hoverTabLabels = document.querySelectorAll(".top-navigation__hover-tabs__label");
			pseudoHoverClass = "hover";
			
			if (hoverTabLabels.length === 0 || hoverTabs.length === 0) { return; }
			
			attachEventListeners();
		}
		
		return { init: init };
	}()),
	
	init: function () {
		this.slideIn.init();
		this.touchHover.init();
	}
};

/* --- SSO Login Status --- */

IS24.core.sso = (function () {
	function getFirstAndSecondLevelDomain() {
		var host = location.hostname,
			hostWithoutFirstLevel,
			secondLevelDotIndex;
		
		// trim leading and trailing dots
		while (host.slice(0,1) === ".") { host = host.slice(1); }
		while (host.slice( -1) === ".") { host = host.slice(0, host.length - 1); }
		
		if (host.indexOf(".") !== -1) {
			hostWithoutFirstLevel = host.substring(0, host.lastIndexOf(".") - 1);
			secondLevelDotIndex = hostWithoutFirstLevel.lastIndexOf(".");
			if (secondLevelDotIndex !== -1) {
				host = host.substring(secondLevelDotIndex + 1);
			}
		}
		
		return host;
	}
	
	function getSSOApiDomain() {
		return "sso." + getFirstAndSecondLevelDomain();
	}
	
	function attachReturnURLs() {
		var loginLink = document.getElementById("link_loginLinkInternal"),
			logoutLink = document.getElementById("link_logoutLinkInternal"),
			currentLocation = encodeURIComponent(location.href),
			ssoAppName = (IS24.ssoAppName) ? encodeURIComponent(IS24.ssoAppName) : undefined;
		
		if (loginLink) {
			loginLink.href += "&returnUrl=" + currentLocation;
			if (IS24.ssoAppName) {
				loginLink.href += "&appName=" + ssoAppName;
			}
		}
		if (logoutLink) {
			logoutLink.href += "&openid.ssoff.rp_post_logout_url=" + currentLocation;
		}
	}
	
	function getJSONPUrl(callbackName) {
		var apiHost = getSSOApiDomain(),
			timestamp = Math.round((new Date()).getTime() / 1000); // cache killer
		
		return "https://" + apiHost + "/sso/getLoginInfo?callback=" + callbackName + "&ts=" + timestamp;
	}
	
	function makeJSONPCall(callbackName) {
		var scriptElement = document.createElement('script');
		scriptElement.type = "text/javascript";
		scriptElement.src = getJSONPUrl(callbackName);
		document.getElementsByTagName('head')[0].appendChild(scriptElement);
	}
	
	function updateLoginStatus() {
		attachReturnURLs();
		makeJSONPCall("IS24.core.sso.jsonpCallback");
	}
	
	function jsonpCallback(data) {
		var loginWrapper = document.querySelector(".sso-login"),
			userNameContainer = document.querySelector(".sso-login__user-name"),
			loggedInClass = "sso-login--logged-in";
		
		if (!(data && data.username) || !loginWrapper || !userNameContainer) { return; }
		
		userNameContainer.innerHTML = data.username;
		IS24.core.helpers.addClass(loginWrapper, loggedInClass);
	}
	
	return {
		updateLoginStatus: updateLoginStatus,
		jsonpCallback: jsonpCallback
	};
}());
